/**
 * Copyright OPS4J
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain
 * a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ops4j.pax.wicket.internal.injection;

import static org.ops4j.pax.wicket.api.ContentSource.APPLICATION_NAME;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.ops4j.pax.wicket.api.ContentAggregator;
import org.ops4j.pax.wicket.api.ContentSource;
import org.ops4j.pax.wicket.api.NoBeanAvailableForInjectionException;
import org.ops4j.pax.wicket.api.PageFactory;
import org.ops4j.pax.wicket.api.PaxWicketApplicationFactory;
import org.ops4j.pax.wicket.api.PaxWicketInjector;
import org.ops4j.pax.wicket.internal.DummyServiceReference;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Filter;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTracker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * We assume that all bundles exporting a service implementing at least one of the following interfaces should also be
 * able to be searched for beans: {@link ContentSource}, {@link ContentAggregator}, {@link PageFactory} and
 * {@link PaxWicketApplicationFactory}.
 */
public class BundleDelegatingComponentInstanciationListener extends ServiceTracker implements
        PaxWicketInjector {

    private static final Logger LOGGER = LoggerFactory.getLogger(BundleDelegatingComponentInstanciationListener.class);

    private static final String FILTER = "(|" +
            "(objectClass=" + ContentSource.class.getName() + ")" +
            "(objectClass=" + ContentAggregator.class.getName() + ")" +
            "(objectClass=" + PageFactory.class.getName() + ")" +
            "(objectClass=" + PaxWicketApplicationFactory.class.getName() + ")" +
            ")";

    private Map<ServiceReference, BundleAnalysingComponentInstantiationListener> listeners;
    private final String applicationName;

    private final String injectionSource;

    public BundleDelegatingComponentInstanciationListener(BundleContext context, String applicationName,
            Bundle paxWicketBundle, String injectionSource) {
        super(context, createFilter(context), null);

        this.applicationName = applicationName;
        this.injectionSource = injectionSource;

        listeners = new HashMap<ServiceReference, BundleAnalysingComponentInstantiationListener>();
        listeners.put(new DummyServiceReference(),
            new BundleAnalysingComponentInstantiationListener(paxWicketBundle.getBundleContext(), injectionSource));

        open(true);
    }

    public void inject(Object toInject) {
        synchronized (listeners) {
            Collection<BundleAnalysingComponentInstantiationListener> values = listeners.values();
            for (BundleAnalysingComponentInstantiationListener analyser : values) {
                if (analyser.injectionPossible(toInject.getClass())) {
                    analyser.inject(toInject);
                    return;
                }
            }
        }
        throw new NoBeanAvailableForInjectionException();
    }

    @Override
    public Object addingService(ServiceReference serviceReference) {
        String appName = (String) serviceReference.getProperty(APPLICATION_NAME);
        if (!applicationName.equals(appName)) {
            LOGGER.debug("Applicationname {} does not match service application name {}", appName, applicationName);
            return null;
        }
        LOGGER.info("Adding bundle {} to DelegatingComponentInstanciationListener",
            serviceReference.getBundle().getSymbolicName());
        Bundle bundle = serviceReference.getBundle();
        synchronized (listeners) {
            listeners.put(serviceReference, new BundleAnalysingComponentInstantiationListener(
                bundle.getBundleContext(), injectionSource));
        }
        return super.addingService(serviceReference);
    }

    @Override
    public void removedService(ServiceReference serviceReference, Object o) {
        String appName = (String) serviceReference.getProperty(APPLICATION_NAME);
        if (!applicationName.equals(appName)) {
            LOGGER.debug("Applicationname {} does not match service application name {}", appName, applicationName);
            return;
        }
        LOGGER.info("Removing bundle {} to DelegatingClassLoader", serviceReference.getBundle().getSymbolicName());
        synchronized (listeners) {
            listeners.remove(serviceReference);
        }
        super.removedService(serviceReference, o);
    }

    private static Filter createFilter(BundleContext context) {
        try {
            return context.createFilter(FILTER);
        } catch (InvalidSyntaxException e) {
            throw new IllegalStateException(
                String.format("Unexpected behavior! The filter %s should not fail", FILTER), e);
        }
    }

}
